"""hotfix_ENA_import_links_20210729

Checking the EMBL-ENA accession codes started to fail on 26 July 2021 (or slightly earlier).
The reason is that the ENA website changed URLs.

This script replaces the used URL with an updated version to restore the functionality.
The hotfix was created for BioNumerics 8.0 and 7.6, it is obsolete for 8.1 and above and may not work on older versions.
"""
#//$autorun;event=PostCreateMainWin;

import bns
import xml.etree.cElementTree as ET


HOTFIX_NAME = 'hotfix_ENA_import_links_20210729'


def HotfixCheckLink(self, accCode):
	"""The corrected ENA accession link check function."""
	url = 'https://www.ebi.ac.uk/ena/browser/api/xml/{0}'.format(accCode)
	response = bns.Util.UrlTools.urlget(url)
	try:
		xml = ET.XML(response)
	except Exception as e:
		# if an XML exception is raised
		raise RuntimeError(response)
	
	if xml.find('RUN'):
		return True
	elif xml.text:
		raise RuntimeError(xml.text.strip())
	else:
		raise RuntimeError(response)


HotfixCheckLink.is_hotfix_method = HOTFIX_NAME  # Mark as hotfix method


if bns.Database.Db.Info.SoftwareVersion <= '7.6':
	def replace_checklink(downloader, new_method):
		# Note: for some reason, this only works on instance level and must replace in multiple places,
		# despite those just importing the same list. Not the case in BN80 and later.
		downloader.CheckLink = lambda accCode: new_method(downloader, accCode)
		# set explicitly for downloaders
		import wgMLST_Client.importSRSLinks.DownloadSites as DS
		idx = DS.downloaderList.index(downloader)
		DS.downloaderList[idx] = downloader
		DS.downloaders[downloader.ID] = downloader
		# set explicitly for import wizard downloaders
		import wgMLST_Client.importSRSLinks.importwizard
		wgMLST_Client.importSRSLinks.importwizard.downloaderList[idx] = downloader
		wgMLST_Client.importSRSLinks.importwizard.downloaders[downloader.ID] = downloader
		# set explicitly for job submission checks as well
		import wgMLST_Client.WgmlstLaunchSingleEntryJobsDlg
		wgMLST_Client.WgmlstLaunchSingleEntryJobsDlg.downloaders[downloader.ID] = downloader
else:
	def replace_checklink(downloader, new_method):
		# Simply changing on the class works
		downloader.__class__.CheckLink = new_method
	
	
def main():
	# make sure the wgMLST_Client plugin is installed
	info = {'name': "", 'description': "", 'filename': "", 'version': "", 'isactive': False}
	bns.Util.Program.GetPluginInfo("wgMLST_Client", byref=info)
	if not info['isactive']:
		return
	# Get the downloader class and instance
	import wgMLST_Client.importSRSLinks.DownloadSites as ds
	downloader_cls = ds.ENA_Downloader
	downloader = ds.downloaders[downloader_cls.ID]
	# Check for previous application of hotfixes
	existing_hotfix = getattr(downloader_cls.CheckLink, 'is_hotfix_method', None)
	if existing_hotfix == HOTFIX_NAME:
		bns.Util.Program.AddProgramLog("The hotfix '{0}' was applied previously, execution aborted.".format(HOTFIX_NAME))
		return
	elif existing_hotfix is not None:
		msg = (
			"The hotfix '{0}' was not applied because a different hotfix of the same functionality is already present ('{1}'). "
		).format(HOTFIX_NAME, existing_hotfix)
		bns.Util.Program.MessageBox('Error', msg, 'exclamation')
		return
	# We only want to replace the method if it is broken,
	# so we do not break any fixed versions and can notify the user about the obsolescence of this script.
	# Therefore we first try the unpatched method and see if it produces a specific error.
	test_accessions = ['SRR1112181', 'ERR6326109']  # we know for sure these accessions exists
	try:
		for acc in test_accessions:
			downloader.CheckLink(acc)
	except bns.Util.UrlTools.HTTPError:
		original_method = downloader_cls.CheckLink
		replace_checklink(downloader, HotfixCheckLink)
		try:
			for acc in test_accessions:
				downloader.CheckLink(acc)
		except Exception as e:
			# Still doesn't work => undo and notify user about faulty hotfix
			replace_checklink(downloader, original_method)
			msg = (
				"The hotfix '{0}' could not fix the functionality. "
				"There are still errors after applying the hotfix, so the fix was undone. "
				"\r\n\r\n{1}: {2}"
			).format(HOTFIX_NAME, e.__class__.__name__, str(e).strip())
			bns.Util.Program.MessageBox('Error', msg, 'exclamation')
		else:
			# Success, do not show popup to user but write to log
			bns.Util.Program.AddProgramLog("The hotfix '{0}' was applied succesfully.".format(HOTFIX_NAME))
	except Exception as e:
		# Unknown exception, don't try to fix something we're not designed for
		msg = (
			"The hotfix '{0}' encountered an unexpected error. "
			"The error appears different from what the hotfix was designed for, so the fix was not applied. "
			"\r\n\r\n{1}: {2}"
		).format(HOTFIX_NAME, e.__class__.__name__, str(e).strip())
		bns.Util.Program.MessageBox('Error', msg, 'exclamation')
	else:
		# Existing method works => hotfix no longer needed for the current program version
		msg = (
			"The hotfix '{0}' seems to be no longer needed. "
			"The current version of the program downloads ENA accessions correctly. "
			"It is recommended to remove the hotfix script. "
		).format(HOTFIX_NAME)
		bns.Util.Program.MessageBox('Obsolete hotfix', msg, 'info')
	

if __name__ == '__main__':
	main()


#
#
# END OF FILE
